"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-explicit-any */
const availabilitySync_1 = __importDefault(require("../lib/availabilitySync"));
const logger_1 = __importDefault(require("../logger"));
const axios_1 = __importDefault(require("axios"));
class JellyfinAPI {
    constructor(jellyfinHost, authToken, deviceId) {
        this.jellyfinHost = jellyfinHost;
        this.authToken = authToken;
        let authHeaderVal = '';
        if (this.authToken) {
            authHeaderVal = `MediaBrowser Client="Overseerr", Device="Axios", DeviceId="${deviceId}", Version="10.8.0", Token="${authToken}"`;
        }
        else {
            authHeaderVal = `MediaBrowser Client="Overseerr", Device="Axios", DeviceId="${deviceId}", Version="10.8.0"`;
        }
        this.axios = axios_1.default.create({
            baseURL: this.jellyfinHost,
            headers: {
                'X-Emby-Authorization': authHeaderVal,
                'Content-Type': 'application/json',
                Accept: 'application/json',
            },
        });
    }
    async login(Username, Password) {
        try {
            const account = await this.axios.post('/Users/AuthenticateByName', {
                Username: Username,
                Pw: Password,
            });
            return account.data;
        }
        catch (e) {
            throw new Error('Unauthorized');
        }
    }
    setUserId(userId) {
        this.userId = userId;
        return;
    }
    async getServerName() {
        try {
            const account = await this.axios.get("/System/Info/Public'}");
            return account.data.ServerName;
        }
        catch (e) {
            logger_1.default.error(`Something went wrong while getting the server name from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('girl idk');
        }
    }
    async getUsers() {
        try {
            const account = await this.axios.get(`/Users`);
            return { users: account.data };
        }
        catch (e) {
            logger_1.default.error(`Something went wrong while getting the account from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('Invalid auth token');
        }
    }
    async getUser() {
        try {
            const account = await this.axios.get(`/Users/${this.userId ?? 'Me'}`);
            return account.data;
        }
        catch (e) {
            logger_1.default.error(`Something went wrong while getting the account from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('Invalid auth token');
        }
    }
    async getLibraries() {
        try {
            const mediaFolders = await this.axios.get(`/Library/MediaFolders`);
            return this.mapLibraries(mediaFolders.data.Items);
        }
        catch (mediaFoldersError) {
            // fallback to user views to get libraries
            // this only affects LDAP users
            try {
                const mediaFolders = await this.axios.get(`/Users/${this.userId ?? 'Me'}/Views`);
                return this.mapLibraries(mediaFolders.data.Items);
            }
            catch (e) {
                logger_1.default.error(`Something went wrong while getting libraries from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
                return [];
            }
        }
    }
    mapLibraries(mediaFolders) {
        const excludedTypes = [
            'music',
            'books',
            'musicvideos',
            'homevideos',
            'boxsets',
        ];
        return mediaFolders
            .filter((Item) => {
            return (Item.Type === 'CollectionFolder' &&
                !excludedTypes.includes(Item.CollectionType));
        })
            .map((Item) => {
            return {
                key: Item.Id,
                title: Item.Name,
                type: Item.CollectionType === 'movies' ? 'movie' : 'show',
                agent: 'jellyfin',
            };
        });
    }
    async getLibraryContents(id) {
        try {
            const contents = await this.axios.get(`/Users/${this.userId}/Items?SortBy=SortName&SortOrder=Ascending&IncludeItemTypes=Series,Movie,Others&Recursive=true&StartIndex=0&ParentId=${id}&collapseBoxSetItems=false`);
            return contents.data.Items.filter((item) => item.LocationType !== 'Virtual');
        }
        catch (e) {
            logger_1.default.error(`Something went wrong while getting library content from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('Invalid auth token');
        }
    }
    async getRecentlyAdded(id) {
        try {
            const contents = await this.axios.get(`/Users/${this.userId}/Items/Latest?Limit=12&ParentId=${id}`);
            return contents.data;
        }
        catch (e) {
            logger_1.default.error(`Something went wrong while getting library content from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('Invalid auth token');
        }
    }
    async getItemData(id) {
        try {
            const contents = await this.axios.get(`/Users/${this.userId}/Items/${id}`);
            return contents.data;
        }
        catch (e) {
            if (availabilitySync_1.default.running) {
                if (e.response && e.response.status === 500) {
                    return undefined;
                }
            }
            logger_1.default.error(`Something went wrong while getting library content from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('Invalid auth token');
        }
    }
    async getSeasons(seriesID) {
        try {
            const contents = await this.axios.get(`/Shows/${seriesID}/Seasons`);
            return contents.data.Items;
        }
        catch (e) {
            logger_1.default.error(`Something went wrong while getting the list of seasons from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('Invalid auth token');
        }
    }
    async getEpisodes(seriesID, seasonID) {
        try {
            const contents = await this.axios.get(`/Shows/${seriesID}/Episodes?seasonId=${seasonID}`);
            return contents.data.Items.filter((item) => item.LocationType !== 'Virtual');
        }
        catch (e) {
            logger_1.default.error(`Something went wrong while getting the list of episodes from the Jellyfin server: ${e.message}`, { label: 'Jellyfin API' });
            throw new Error('Invalid auth token');
        }
    }
}
exports.default = JellyfinAPI;
